﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.Entity;
using System.ComponentModel.DataAnnotations;
using System.Data.Entity.ModelConfiguration;

namespace NerdDinnerReloaded.Models
{
    //
    // Our POCO model classes. Note that these could be implemented in a separate class
    // library project.  They have no dependency on ASP.NET MVC or the EF Code-First Persistence
    // library.

    public class Address
    {
        [Required]
        public string   Street      { get; set; }

        [Required]
        public string   City        { get; set; }

        [Required]
        public string   PostCode    { get; set; }

        [Required]
        public string   Country     { get; set; }
    }

    public class Dinner
    {
        public int DinnerID { get; set; }

        [Required(ErrorMessage = "Please enter a Dinner Title")]
        [StringLength(20, ErrorMessage = "Title is too long")]
        public string Title { get; set; }

        [Required(ErrorMessage = "Please enter the Date of the Dinner")]
        public DateTime EventDate { get; set; }

        public Address Address { get; set; }

        [Required(ErrorMessage = "Please enter your email address")]
        [RegularExpression(".+\\@.+\\..+", ErrorMessage = "Please enter a valid email address")]
        public string HostedBy { get; set; }

        public virtual ICollection<RSVP> RSVPs { get; set; }
    }

    public class RSVP
    {
        public int     RsvpID         { get; set; }
        public int     DinnerID       { get; set; }
        public string  AttendeeEmail  { get; set; }

        public virtual Dinner Dinner  { get; set; }
    }

    // 
    // Our EF Code First Context class.  This class handles persistence and
    // interactions with the database.  This could have also been implemented
    // in a separate class library.

    public class NerdDinners : DbContext
    {
        public DbSet<Dinner> Dinners { get; set; }
        public DbSet<RSVP>   RSVPs   { get; set; }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            // Register Address Class as a ComplexType
            modelBuilder.ComplexType<Address>();

            // Map Dinner class to a legacy tblDinners table
            modelBuilder.Entity<Dinner>().MapSingleType(dinner => new {
                colID = dinner.DinnerID,
                colTitle = dinner.Title,
                colDate = dinner.EventDate,
                colAddress_Street = dinner.Address.Street,
                colAddress_City = dinner.Address.City,
                colAddress_Zip = dinner.Address.PostCode,
                colAddress_Country = dinner.Address.Country,
                colHost = dinner.HostedBy,
            }).ToTable("tblDinners");
        }
    }
}